#include "../../../smilepot.h"

const smilePotential* pot=0;  // single instance of potential
const double* pointx=0, *pointy=0, *pointz=0, *pointm=0;
int npoints=0;

/// store particle positions and masses used to initialize potential
int set_particles(double x[], double y[], double z[], double m[], int n)
{
    pointx=x;
    pointy=y;
    pointz=z;
    pointm=m;
    npoints=n;
    return x && y && z && m && n ? 0 : -1;
}

/// create the instance of potential
int initialize_code(int nparams, char** params)
{
    if(nparams<=0) return -2;
    if(pot) return -3;
    if(npoints>0 && pointx!=0 && pointy!=0 && pointz!=0 && pointm!=0)
        pot=smilepot_create_from_particles(nparams, params, npoints, pointx, pointy, pointz, pointm);
    else
        pot=smilepot_create_params(nparams, params);
    if(!pot) return -1;
    return 0;
}

/// destroy the instance of potential
int cleanup_code()
{
    if(!pot) return -1;
    smilepot_delete(pot);
    return 0;
}

/// compute accelerations at given points: x,y,z are input coordinates, ax,ay,az are output accelerations 
int get_gravity_at_point(double eps[],
    double x[], double y[], double z[],
    double ax[], double ay[], double az[], int npoints)
{
    if(!pot) return -1;
    double xyz[3], acc[3];
    for(int i=0; i<npoints; i++) {
        xyz[0]=x[i];
        xyz[1]=y[i];
        xyz[2]=z[i];
        smilepot_force(pot, xyz, acc);
        ax[i]=acc[0];
        ay[i]=acc[1];
        az[i]=acc[2];
    }
    return 0;
}

/// compute potential at given points: x,y,z are input coordinates, p is output potential
int get_potential_at_point(double eps[],
    double x[], double y[], double z[],
    double p[], int npoints)
{
    if(!pot) return -1;
    double xyz[3];
    for(int i=0; i<npoints; i++) {
        xyz[0]=x[i];
        xyz[1]=y[i];
        xyz[2]=z[i];
        p[i]=smilepot_potential(pot, xyz);
    }
    return 0;
}

